/**
* TicTacToePlayer class
* 
* Provides basic strategy for a computer-based player in a game of
* Tic-Tac-Toe. The strategy plays to win or block if possible, but
* will not necessarily choose the best move if a winning or blocking
* move is not available.
* 
* Universal parameters for all methods:
* 
*   p_grid:Array - Nine-element, zero-indexed array representing game
*   grid. Element [0] represents the upper-left square of the game board.
*   Each element may have the value 0 (no mark), 1 (opponent's mark), or
*   2 (computer's mark).
* 
*   p_move:Number - Square (0-8) in which a mark is to be made or
*   evaluated.
* 
*   p_player:Number - 1 for opponent, 2 for computer player.
*/

class gov.ftc.squaresgame.TicTacToePlayer extends Object {

	private var __wins:Array = new Array();
	private var __tempMove:Number;

/**
* Defines all combinations of squares in p_grid that represent
* "Tic-Tac-Toe."
*/

	public function TicTacToePlayer() {
		__wins = [[0,1,2],[3,4,5],[6,7,8],[0,3,6],[1,4,7],[2,5,8],[0,4,8],[2,4,6]];
	}

/**
* Returns the grid index of the computer's next move in a given game 
* as described by p_grid. Calls findBestMove() to win, then calls
* findBestMove() to block if no win is possible, then calls 
* findValidMove() if no block is possible.
*/

	public function findMove(p_grid:Array):Number {
		// input parameter is flat array, zero-indexed
		// 0 indicates empty unused space, 1 indicates opponent mark, 2 indicates computer mark,
		// 3 indicates empty but previously selected space
		// returns index of space representing the best move

//		trace("TicTacToePlayer.findMove received grid: " + p_grid);

		var nextMove:Number = findBestMove(p_grid, true);
		
		if (nextMove == -1) nextMove = findBestMove(p_grid, false);
		
		if (nextMove == -1) {
			nextMove = findValidMove(p_grid);
		}
//		trace("TicTacToePlayer.findMove returned: " + nextMove);
		return nextMove;
	}

/**
* Returns array of winning marks if p_move would result in a win for p_player in a given
* game described by p_grid, and null otherwise.
* 
* Note: This implementation considers five marks a "win."
*/

	public function getWinMask(p_grid:Array, p_move:Number, p_player:Number):Array {
		p_grid[p_move] = p_player;

		var countMarks:Number = 0;
		var winCountArray:Array = [0,0,0,0,0,0,0,0,0];
		var j:Number = p_grid.length;
		while (j--) {
			if (p_grid[j] == p_player) {
				countMarks++;
				winCountArray[j] = 1;
			}
			if (countMarks == 5) return winCountArray;
		}

		var winRowArray:Array = [0,0,0,0,0,0,0,0,0];
		var i:Number = __wins.length;
		while (i--) {
			var r:Array = __wins[i];
			if (p_grid[r[0]] == p_player && p_grid[r[1]] == p_player && p_grid[r[2]] == p_player) {
				winRowArray[r[0]] = 1;
				winRowArray[r[1]] = 1;
				winRowArray[r[2]] = 1;
				return winRowArray;
			}
		}
		return null;
	}




/**
* Used by findMove(), searches p_grid for a winning move if p_toWin is true,
* or for a blocking move if p_toWin is false. Returns the first move found
* or -1 if no such move is possible.
*/
	
	private function findBestMove(p_grid:Array, p_toWin:Boolean):Number {
//		trace("TicTacToePlayer.findBestMove: " + p_toWin);
		var pl:Number = (p_toWin) ? 2 : 1;
		var i:Number = __wins.length;
		while(i--) {
			var r:Array = __wins[i];
			var tic:Number = p_grid[r[0]];
			var tac:Number = p_grid[r[1]];
			var toe:Number = p_grid[r[2]];
			if (tic == pl && tac == pl && (toe == 0 || toe == 3)) {
				return r[2];
			} else if (tic == pl && toe == pl && (tac == 0 || tac == 3)) {
				return r[1];
			} else if (toe == pl && tac == pl && (tic == 0 || tic == 3)) {
				return r[0];
			}
		}
		return -1;
	}
	
/**
* Used by findMove(), returns a random valid move (unoccupied square) given
* a game described by p_grid.
*/
	
	private function findValidMove(p_grid:Array):Number {
		__tempMove = Math.floor(Math.random() * 9);
//		trace("TicTacToePlayer.findValidMove on grid: " + p_grid + ", trying " +__tempMove);
		if (p_grid[__tempMove] != 0 && p_grid[__tempMove] != 3) {
			findValidMove(p_grid);
		}
		return __tempMove;
	}
}
